/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/





//created by Andrew Caldwell on 12/12/99 caldwell@cs.ucla.edu

#ifndef _KPLACESPLITTER_H__
#define _KPLACESPLITTER_H__

#include "baseBlkSplitter.h"

/*
  the KPlaceSplitter should be called after using a global placer
  on Capo.  It uses the locations assigned to the cells in the block
  to split the block.  If the global placer is not called first,
  all the cells will be in the middle of the block, and the first
  step is useless.

  This first version is only for V-splitting of blocks

  The process is as follows:
  1) calculate the (area) weighted center of mass for the cells in 
	the block.  If it is not within the middle 50% of the block
	(within 25%-75% of the block's x and y span), then
	the placement is assumed to not have enough information
	to be useful, and the block is split w/ a partitioning call.
  2) generate a linear ordering from the placement
  3) iterate through linear ordering, looking for a partitioning s.t.
	-cells with the same X-location end up in the same partition
	-at least 40% of cell area is in each partition
	-partitioning has the minimum cut of all such partitions

  4) if this fails (i.e. cells are all located at the same x-coord),
	call the partitioner.
*/

class KPlaceSplitter : public BaseBlockSplitter
{
    const Placement&		_origPl;
    const HGraphWDimensions&	_netlistHG;

  public:

    KPlaceSplitter(CapoBlock&  blockToSplit,
		   const CapoPlacer& capo,
		   const Placement&  origPl,
		   CapoSplitterParams params);

    virtual ~KPlaceSplitter(){}

    vector<CapoBlock*>& getNewBlocks() {return _newBlocks;}
}; 


#endif
